/* Copyright 2006, freshmowed software.  All rights reserved. */

/* Example trackerData XML
<countabout xmlns:fmctbt="http://www.freshmowed.com/namespaces/countabout" id="uri:freshmowed.com/123456/545566" hostID="41a36cb49284855e">
<trackerData>
<title>Calories Consumed</title>
<units prefix="true">kcal</units>
<chart></chart>  // see chart XML sample below
<dataSet></dataSet> // see dataSet XML sample below
</trackerData>
<trackerData ...>
</trackerData>
</countabout>
*/

CountaboutData = function()
{
    this.trackerData = new Array();
    
    this.trackerData.push(new TrackerData("Countabout"));
    this.trackerData.push(new TrackerData("Countabout Two"));
    this.trackerData.push(new TrackerData("Countabout Three"));
}

CountaboutData.prototype =
{
    extractFromDOM: function(XMLDOMObject, mode)  // mode = "replace" (default), "merge-keep-existing", "merge-use-incoming"
    {
        if ( mode == null ) mode = "replace";
        
        var aDOM = XMLDOMObject.getElementsByTagName("countabout");
        if ( aDOM != null )
        {
          var countaboutDOM = aDOM.item(0);
          
          var anElement = countaboutDOM.getAttributeNode("id");
          this.ID = anElement ? anElement.nodeValue : "unknown";
          anElement = countaboutDOM.getAttributeNode("hostID");
          this.hostID = anElement ? anElement.nodeValue : "unknown";

          var trackers = countaboutDOM.getElementsByTagName("trackerData");
          for ( i = 0; trackers != null && i < trackers.length; i++ )
          {
              var aTracker = new TrackerData()
              aTracker.extractFromDOM(trackers.item(i));
              this.trackerData[i] = aTracker;
          }

        }
        else
        {
          aDOM = XMLDOMObject.getElementsByTagName("countaboutConfig");
          if ( aDOM != null )
          {
            var configNodes = aDOM.getElementsByTagName("trackerConfig");
            for ( i = 0; configNodes != null && configNodes.length > 0 && i < configNodes.length; i++ )
            {
                var aTracker = this.trackerData[i] != null ? this.trackerData[i] : new TrackerData()
                aTracker.extractFromConfigDOM(configNodes.item(i));
                this.trackerData[i] = aTracker;
            }
          }
          else
          {
            aDOM = XMLDOMObject.getElementsByTagName("countaboutData");
            if ( aDOM != null )
            {
              var dataSetNodes = aDOM.getElementsByTagName("dataSet");
              for ( i = 0; dataSetNodes != null && dataSetNodes.length > 0 && i < dataSetNodes.length; i++ )
              {
                  var aTracker = this.trackerData[i] != null ? this.trackerData[i] : new TrackerData()
                  var aDataSet = aTracker.dataSet;
                  aDataSet.extractFromDOM(dataSetNodes.item(i));
                  this.trackerData[i] = aTracker;
              }
            }
            else
            {
              alert("CountaboutData.extractFromDOM: unknown XML format.")
            }
            
          }
        }

        return this;
    },

    toXML: function()
    {
        var result = "<countabout xmlns:fmctbt='http://www.freshmowed.com/namespaces/countabout'";
        if ( this.ID != null && this.ID != "unknown" ) result += " id=\"" + this.ID + "\"";
        if ( this.hostID != null && this.hostID != "unknown" ) result += " hostID=\"" + this.hostID + "\"";
        result += ">";
        for ( var i = 0; i < this.trackerData.length; i++ )
        {            
            result += this.trackerData[i].toXML();
        }
        result += "</countabout>";

        return result;
    },
    
    toConfigXML: function()
    {
      var result = "<countaboutConfig xmlns:fmctbt='http://www.freshmowed.com/namespaces/countabout'";
      if ( this.ID != null && this.ID != "unknown" ) result += " sourceID=\"" + this.ID + "\"";
      result += ">";
      for ( var i = 0; i < this.trackerData.length; i++ )
      {            
          result += this.trackerData[i].toTrackerConfigXML();
      }
      result += "</countaboutData>";

      return result;
    },
    
    toDataXML: function()
    {
      var result = "<countaboutData xmlns:fmctbt='http://www.freshmowed.com/namespaces/countabout'";
      if ( this.ID != null && this.ID != "unknown" ) result += " sourceID=\"" + this.ID + "\"";
      result += ">";
      for ( var i = 0; i < this.trackerData.length; i++ )
      {            
          result += this.trackerData[i].toTrackerDataXML();
      }
      result += "</countaboutData>";

      return result;
    },
    
    fromXML: function(xml)
    {
      var parser = new DOMImplementation();
      this.extractFromDOM(parser.loadXML(xml));
    },

    sampleXML: function()
    {
        var todayUTC = Utilities.daysAgoUTC(0);
        var todayDaysSinceEpoch = Utilities.numDaysSinceEpoch(todayUTC);

        return   "<countabout xmlns=\"http://www.freshmowed.com/namespaces/tracker\" " + 
                 "id=\"uri:freshmowed.com/123456/545566\" hostID=\"41a36cb49284855e\">\n" +
                 "<trackerData>\n" +
                 "<title>Weight</title>\n" + 
                 "<enabled>true</enabled>\n" +
                 "<units prefix=\"false\">lbs</units>\n" +
                 "<additionMode>replace</additionMode>" +
                 "<statKeys>statAvg,statMin,statMax,statDiff,statTarget</statKeys>" +
                 "<chart id=\"1234\" type=\"bar\">\n" + 
                 "  <xDataType>history</xDataType>\n" + 
                 "  <yMin>150</yMin>\n" + 
                 "  <goal>50</goal>\n" +
                 "</chart>\n" + 
                 "<dataSet>\n" + 
                 "  <pt>\n" + 
                 "    <y>190</y>\n" + 
                 "    <x>" + (todayDaysSinceEpoch - 1) + "</x>\n" + 
                 "    <xD>" + Utilities.formattedDateString(todayUTC - 1 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                 "    <note>This is my peak for the year</note>\n" + 
                 "    <date>13483499</date>\n" + 
                 "    <by>84398</by>\n" + 
                 "    <ptStyle>\n" + 
                 "      <color>red</color>\n" + 
                 "    </ptStyle>\n" + 
                 "  </pt>\n" + 
                 "  <pt>\n" + 
                 "    <y>188</y>\n" + 
                 "    <x>" + (todayDaysSinceEpoch - 3)  + "</x>\n" + 
                 "    <xD>" + Utilities.formattedDateString(todayUTC - 3 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                 "  </pt>\n" + 
                 "  <pt>\n" + 
                 "    <y>190</y>\n" + 
                 "    <x>" + (todayDaysSinceEpoch - 4)  + "</x>\n" + 
                 "    <xD>" + Utilities.formattedDateString(todayUTC - 4 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                 "  </pt>\n" + 
                 "  <pt>\n" + 
                 "    <y>189</y>\n" + 
                 "    <x>" + (todayDaysSinceEpoch - 6)  + "</x>\n" + 
                 "    <xD>" + Utilities.formattedDateString(todayUTC - 6 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                 "  </pt>\n" + 
                 "  <pt>\n" + 
                 "    <y>194</y>\n" + 
                 "    <x>" + (todayDaysSinceEpoch - 7)  + "</x>\n" + 
                 "    <xD>" + Utilities.formattedDateString(todayUTC - 7 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                 "  </pt>\n" + 
                 "  <pt>\n" + 
                 "    <y>196</y>\n" + 
                 "    <x>" + (todayDaysSinceEpoch - 15)  + "</x>\n" + 
                 "    <xD>" + Utilities.formattedDateString(todayUTC - 15 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                 "  </pt>\n" + 
                 "  <pt>\n" + 
                 "    <y>203</y>\n" + 
                 "    <x>" + (todayDaysSinceEpoch - 28)  + "</x>\n" + 
                 "    <xD>" + Utilities.formattedDateString(todayUTC - 28 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                 "  </pt>\n" + 
                 "</dataSet>\n" +
                 "</trackerData>\n" +
                 "<trackerData>\n" + 
                    "<title>Calories</title>\n" + 
                    "<enabled>true</enabled>\n" +
                    "<units prefix=\"false\">kcal</units>\n" +
                    "<additionMode>add</additionMode>" +
                    "<statKeys>statAvg,statMin,statMax,statTarget</statKeys>" +
                    "<chart id=\"1234\" type=\"bar\">\n" + 
                    "  <xDataType>history</xDataType>\n" + 
                    "  <yMin>1000</yMin>\n" + 
                    "  <goal>2200</goal>\n" +
                    "</chart>\n" + 
                    "<dataSet>\n" + 
                    "  <pt>\n" + 
                    "    <y>3030</y>\n" + 
                    "    <x>" + (todayDaysSinceEpoch - 1) + "</x>\n" + 
                    "    <xD>" + Utilities.formattedDateString(todayUTC - 1 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                    "  </pt>\n" + 
                    "  <pt>\n" + 
                    "    <y>2820</y>\n" + 
                    "    <x>" + (todayDaysSinceEpoch - 3)  + "</x>\n" + 
                    "    <xD>" + Utilities.formattedDateString(todayUTC - 3 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                    "  </pt>\n" + 
                    "  <pt>\n" + 
                    "    <y>2390</y>\n" + 
                    "    <x>" + (todayDaysSinceEpoch - 4)  + "</x>\n" + 
                    "    <xD>" + Utilities.formattedDateString(todayUTC - 4 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                    "  </pt>\n" + 
                    "  <pt>\n" + 
                    "    <y>2500</y>\n" + 
                    "    <x>" + (todayDaysSinceEpoch - 7)  + "</x>\n" + 
                    "    <xD>" + Utilities.formattedDateString(todayUTC - 7 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                    "  </pt>\n" + 
                    "  <pt>\n" + 
                    "    <y>1990</y>\n" + 
                    "    <x>" + (todayDaysSinceEpoch - 15)  + "</x>\n" + 
                    "    <xD>" + Utilities.formattedDateString(todayUTC - 15 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                    "  </pt>\n" + 
                    "  <pt>\n" + 
                    "    <y>3450</y>\n" + 
                    "    <x>" + (todayDaysSinceEpoch - 28)  + "</x>\n" + 
                    "    <xD>" + Utilities.formattedDateString(todayUTC - 28 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                    "  </pt>\n" + 
                    "</dataSet>\n" +
                    "</trackerData>\n" +
                    "<trackerData>\n" + 
                       "<title>Carbohydrates</title>\n" + 
                       "<enabled>true</enabled>\n" +
                       "<units prefix=\"false\">carbs</units>\n" +
                       "<additionMode>add</additionMode>" +
                       "<statKeys>statAvg,statMedian,statMax,statTarget</statKeys>" +
                       "<chart id=\"1234\" type=\"bar\">\n" + 
                       "  <xDataType>history</xDataType>\n" + 
                       "  <yMin>0</yMin>\n" + 
                       "  <goal>60</goal>\n" +
                       "</chart>\n" + 
                       "<dataSet>\n" + 
                       "  <pt>\n" + 
                       "    <y>83</y>\n" + 
                       "    <x>" + (todayDaysSinceEpoch - 1) + "</x>\n" + 
                       "    <xD>" + Utilities.formattedDateString(todayUTC - 1 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                       "  </pt>\n" + 
                       "  <pt>\n" + 
                       "    <y>112</y>\n" + 
                       "    <x>" + (todayDaysSinceEpoch - 3)  + "</x>\n" + 
                       "    <xD>" + Utilities.formattedDateString(todayUTC - 3 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                       "  </pt>\n" + 
                       "  <pt>\n" + 
                       "    <y>49</y>\n" + 
                       "    <x>" + (todayDaysSinceEpoch - 4)  + "</x>\n" + 
                       "    <xD>" + Utilities.formattedDateString(todayUTC - 4 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                       "  </pt>\n" + 
                       "  <pt>\n" + 
                       "    <y>250</y>\n" + 
                       "    <x>" + (todayDaysSinceEpoch - 7)  + "</x>\n" + 
                       "    <xD>" + Utilities.formattedDateString(todayUTC - 7 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                       "  </pt>\n" + 
                       "  <pt>\n" + 
                       "    <y>92</y>\n" + 
                       "    <x>" + (todayDaysSinceEpoch - 15)  + "</x>\n" + 
                       "    <xD>" + Utilities.formattedDateString(todayUTC - 15 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                       "  </pt>\n" + 
                       "  <pt>\n" + 
                       "    <y>134</y>\n" + 
                       "    <x>" + (todayDaysSinceEpoch - 28)  + "</x>\n" + 
                       "    <xD>" + Utilities.formattedDateString(todayUTC - 28 * Utilities.MILLISECS_PER_DAY) + "</xD>\n" + 
                       "  </pt>\n" + 
                       "</dataSet>\n" +
                       "</trackerData>\n" +
                    "</countabout>\n";

    }
}

TrackerData = function(aTitle)
{
    this.ID = null;
    this.hostID = null;
    this.title = aTitle;
    this.units = "times";
    this.prefixUnits = false;
    this.additionMode = "replace";
    this.enabled = true;
    this.statsEnabled = new Array("statAvg", "statTarget", "statMin", "statMax");
    this.chart = new Chart();
    this.dataSet = new DataSet();
}

TrackerData.prototype =
{
  extractFromDOM: function(XMLDOMObject)
  {
    this.extractConfig(XMLDOMObject);

    var dataSetNodes = XMLDOMObject.getElementsByTagName("dataSet");
    if ( dataSetNodes && dataSetNodes.length > 0 ) this.dataSet.extractFromDOM(dataSetNodes.item(0));

    return this;
  },
  
  extractFromConfigDOM: function(XMLDOMObject)
  {
    this.extractConfig(XMLDOMObject);
  },
  
  extractFromDataDOM: function(XMLDOMObject)
  {    
    var dataSetNodes = XMLDOMObject.getElementsByTagName("dataSet");
    if ( dataSetNodes && dataSetNodes.length > 0 ) this.dataSet.extractFromDOM(dataSetNodes.item(0));

    return this;
  },

  extractConfig: function(XMLDOMObject)
  {
    var anElement = XMLDOMObject.getElementsByTagName("title");
    this.title = (anElement && anElement.length > 0 && anElement.item(0).firstChild) ? anElement.item(0).firstChild.nodeValue : "";

    anElement = XMLDOMObject.getElementsByTagName("enabled");
    this.enabled = (anElement && anElement.length > 0 && anElement.item(0).firstChild) ? anElement.item(0).firstChild.nodeValue == "true": false;

    var nodes = XMLDOMObject.getElementsByTagName("units");
    if ( nodes && nodes.length > 0 )
    {
        var unitsNode = nodes.item(0);
        this.units = unitsNode.firstChild ? unitsNode.firstChild.nodeValue : null;
        var anAttribute = unitsNode.getAttributeNode("prefix");
        this.prefixUnits = (anAttribute != null && anAttribute.nodeValue == "true") ? true : false;
    }
    else
    {
        this.units = "";
        this.prefixUnits = false;
    }

    anElement = XMLDOMObject.getElementsByTagName("additionMode");
    if ( anElement && anElement.length > 0 && anElement.item(0).firstChild )
    {
      this.additionMode = anElement.item(0).firstChild.nodeValue;

      if ( this.additionMode == "true" ) 
        this.additionMode = "add";
      else if ( this.additionMode == "false" )
        this.additionMode = "replace";
    }

    anElement = XMLDOMObject.getElementsByTagName("statKeys");
    var statKeys = (anElement && anElement.length > 0 && anElement.item(0).firstChild) ? anElement.item(0).firstChild.nodeValue : "statAvg,statMin,statMax,statDiff,statTarget";
    this.statsEnabled = statKeys.split(",");

    this.chart.extractFromDOM(XMLDOMObject.getElementsByTagName("chart").item(0));
  },

  // backwards compatibiity -- original form
  toXML: function()
  {
    var result = "<trackerData>";

    result += this.toConfigXML();
    result += this.dataSet.toXML();

    result += "</trackerData>";

    return result;
  },

  toTrackerDataXML: function()
  {
    return this.dataSet.toXML();
  },
  
  toTrackerConfigXML: function()
  {
    var result = "<trackerConfig>";

    result += this.toConfigXML();

    result += "</trackerConfig>";

    return result;
  },
  
  toConfigXML: function()
  {
    var result = "<title><![CDATA[" + this.title + "]]></title>";
    result += "<enabled>" + this.enabled + "</enabled>";
    
    result += "<units";
    if ( this.prefixUnits )
      result += " prefix=\"true\"><![CDATA[";
    else
      result += "><![CDATA[";
    result += this.units + "]]></units>";
    result += "<additionMode>" + this.additionMode + "</additionMode>";
    result += "<statKeys>" + this.statsEnabled.join() + "</statKeys>";
    result += this.chart.toXML();
    
    return result;
  }
    
};

/*  Example chart XML
<chart id=1234 type="bar">
  <xDataType>history</xDataType>
  <yMin>0</yMin>  // optional, default is zero
  <goal>30</goal>  // can be many
  <style>
    <colorScheme>purple</colorScheme>
  </style>
</chart>
*/
Chart = function(aDOMObject)
{
  this.chartDOM = null;
  this.ID = null;
  this.type = "bar";
  this.yMin = null;
  this.xDataType = null;
  this.yDataType = null;
  this.goal = null;
  this.enabled = true;
  this.styleKeys = new Array();
  this.styleAttributes = new Array(); // associative array, keys are style names (ex: "colorScheme"), also kept in styleKeys array
}

Chart.prototype =
{
  extractFromDOM: function(XMLDOMObject)
  {
      this.chartDOM = XMLDOMObject.getElementsByTagName("chart").item(0);
      var anElement = this.chartDOM.getAttributeNode("id");
      this.ID = anElement ? anElement.nodeValue : "unknown";
      anElement = this.chartDOM.getAttributeNode("type");
      this.type = anElement ? anElement.nodeValue : "bar";

      anElement = this.chartDOM.getElementsByTagName("yMin");
      if ( anElement && anElement.length > 0 && anElement.item(0).firstChild )
      {
          var aYMin = anElement.item(0).firstChild.nodeValue;
          if ( aYMin != null ) 
          {
            this.yMin = parseFloat(aYMin);
            if ( isNaN(this.yMin) ) this.yMin = null;
          }
      }

      anElement = this.chartDOM.getElementsByTagName("xDataType");
      this.xDataType = (anElement && anElement.length > 0 && anElement.item(0).firstChild) ? anElement.item(0).firstChild.nodeValue : null;

      anElement = this.chartDOM.getElementsByTagName("yDataType");
      this.yDataType = (anElement && anElement.length > 0 && anElement.item(0).firstChild) ? anElement.item(0).firstChild.nodeValue : null;

      anElement = this.chartDOM.getElementsByTagName("enabled");
      
      if (anElement && anElement.length > 0 && anElement.item(0).firstChild && anElement.item(0).firstChild.nodeValue == "false") 
        this.enabled = false;

      anElement = this.chartDOM.getElementsByTagName("goal");
      if ( anElement && anElement.length > 0 && anElement.item(0).firstChild )
      {
          var aGoal = anElement.item(0).firstChild.nodeValue;
          if ( aGoal != null ) 
          {
            this.goal = parseFloat(aGoal);
            if ( isNaN(this.goal) ) this.goal = null;
          }
      }

      var DOMstyles = this.chartDOM.getElementsByTagName("style");
      var aStyleNode = (DOMstyles == null || DOMstyles.length == 0 ) ? null : DOMstyles.item(0).firstChild;
      while ( aStyleNode != null )
      {
        this.styleKeys.push(aStyleNode.nodeName);
        this.styleAttributes[aStyleNode.nodeName] = aStyleNode.firstChild.nodeValue;
        aStyleNode = aStyleNode.nextSibling;
      }

      return this;
  },

  toXML: function()
  {
     var result = "<chart";
     if ( this.ID != null && this.ID != "unknown" ) result += " id=\"" + this.ID + "\"";
     if ( this.type != null ) result += " type=\"" + this.type + "\"";
     result += ">";

     if ( !this.enabled ) result += "  <enabled>false</enabled>";
     if ( this.yMin != null ) result += "  <yMin>" + this.yMin + "</yMin>";
     if ( this.xDataType != null ) result += "  <xDataType>" + this.xDataType + "</xDataType>";
     if ( this.yDataType != null ) result += "  <yDataType>" + this.yDataType + "</yDataType>";
     if ( this.goal != null ) result += "  <goal>" + this.goal + "</goal>";

     if ( this.styleKeys != null && this.styleKeys.length > 0 )
     {
        result += "  <style>"
        for ( var i = 0; i < this.styleKeys.length; i++)
        {
            var aKey = this.styleKeys[i];
            result += "    <" + aKey + ">" + this.styleAttributes[aKey] + "</" + aKey + ">";
        }
        result += "  </style>";
     }

    result += "</chart>";

    return result;
  }

};

/*  Example dataSet XML
<dataSet>
<pt>...</pt>  // see pt XML sample below
<pt>...</pt>
</dataSet>
*/
DataSet = function()
{
    this.hasTimeData = null;
    this.points = new Array();
}

DataSet.prototype =
{

    isTimeData: function()
    {
        if ( this.hasTimeData != null ) return this.hasTimeData;
        
        for ( var i = 0; i < this.points.length; i++ )
        {
            if ( this.points[i] == null ||
                this.points[i].yValue == null ||
                this.points[i].xValue == null ||
                this.points[i].yDescription == null ) continue;
            
            if ( this.points[i].yDescription.indexOf(":") != -1 )
            {
                this.hasTimeData = true;
                break;
            }
        }
        
        return this.hasTimeData;
    },

    extractFromDOM: function(XMLDOMObject)
    {
        var pointNodes = XMLDOMObject.getElementsByTagName("pt");
        if ( pointNodes == null || pointNodes.length == 0 )
        {
            return this;  // empty points array already created in constructor
        }
        
        var aPointNode = pointNodes.item(0);
        while ( aPointNode != null )
        {
            var aPoint = new DataPoint();
            aPoint.extractFromDOM(aPointNode);
            this.points.push(aPoint);
            aPointNode = aPointNode.nextSibling;
        }
        
        this.points.sort(this.compare);
        return this;
    },
    
    sort: function()
    {
      this.points.sort(this.compare);
    },
    
    compare: function(a,b) 
    {
         return a.xValue - b.xValue;

    },
    
    removePoint: function(aPoint)
    {
      if ( aPoint == null ) return;
      
      var newArray = new Array();
      var iA;
      for ( iA = 0; iA <= this.points.length; iA++ )
      {
          if ( this.points[iA] && this.points[iA] == aPoint ) continue;
               
          newArray.push(this.points[iA]);
      }
      
      this.points = newArray;
    },

    toXML: function()
    {
        var result = "<dataSet>";
        for ( var i = 0; this.points && i < this.points.length; i++ )
        {
          if ( !this.points[i] ) continue;
          
          if ( !this.points[i].xValue || !this.points[i].yValue ) continue;

          if ( isNaN(this.points[i].yValue) ||
               isNaN(this.points[i].xValue) ) 
               {
                 alert("point " + i + " is NaN");
                 continue;
               }

            if ( this.points[i].xValue == null || ( this.points[i].yValue == null && this.points[i].note == null ) )
              continue;
            
            result += this.points[i].toXML();
        }
        result += "</dataSet>";

        return result;
    },
    
    indexOfPoint: function(aPoint)
    {
      for ( var i = 0; i < this.points.length; i++)
      {
          if ( this.points[i] == aPoint ) 
          {
            return i;
          }
      }
          
      return -1;
    },
    
    pointForX: function(anXValue)
    {
        
        for ( var i = 0; i < this.points.length; i++)
        {
            if ( this.points && this.points[i] && this.points[i].xValue == anXValue ) 
            {
              return this.points[i];
            }
        }
            
        return null;
    },
    
    averageForRange: function(minX, maxX)
    {
        var count = 0;
        var sum = 0.0;
        
        for ( var i = 0; i < this.points.length; i++ )
        {
            if ( this.points[i] == null ||
                 this.points[i].yValue == null ||
                 this.points[i].xValue == null ||
                 isNaN(this.points[i].yValue) ||
                 isNaN(this.points[i].xValue) ) continue;
          
             if ( this.points[i].xValue < minX ||
                  this.points[i].xValue > maxX ) continue;

            count++;
            sum += this.points[i].yValue;
        }
        
        if (count == 0) return null;
        
        var avg = sum / count;
        return avg;
    }, 
    
    totalForRange: function(minX, maxX)
    {
        var count = 0;
        var sum = 0.0;
        
        for ( var i = 0; i < this.points.length; i++ )
        {
            if ( this.points[i] == null ||
                 this.points[i].yValue == null ||
                 this.points[i].xValue == null ||
                 isNaN(this.points[i].yValue) ||
                 isNaN(this.points[i].xValue) ) continue;
          
             if ( this.points[i].xValue < minX ||
                  this.points[i].xValue > maxX ) continue;

            count++;
            sum += this.points[i].yValue;
        }
        
        if (count == 0) return null;
        
        return sum;
    }, 
    
    minInRange: function(minX, maxX)
    {
        var found = false;
        var currentMin = 99999999;
      
        for ( var i = 0; i < this.points.length; i++ )
        {
            if ( this.points[i] == null ||
                 this.points[i].yValue == null ||
                 this.points[i].xValue == null ||
                 isNaN(this.points[i].yValue) ||
                 isNaN(this.points[i].xValue) ) continue;
        
             if ( this.points[i].xValue < minX ||
                  this.points[i].xValue > maxX ) continue;

            if ( this.points[i].yValue < currentMin ) 
            {
              currentMin = this.points[i].yValue;
              found = true;
            }
        }
      
        if (!found) return null;
      
        return currentMin;
    },
    
    maxInRange: function(minX, maxX)
    {
        var found = false;
        var currentMax = -99999999;
      
        for ( var i = 0; i < this.points.length; i++ )
        {
            if ( this.points[i] == null ||
                 this.points[i].yValue == null ||
                 this.points[i].xValue == null ||
                 isNaN(this.points[i].yValue) ||
                 isNaN(this.points[i].xValue) ) continue;
        
             if ( this.points[i].xValue < minX ||
                  this.points[i].xValue > maxX ) continue;

            if ( this.points[i].yValue > currentMax ) 
            {
              currentMax = this.points[i].yValue;
              found = true;
            }
        }
      
        if (!found) return null;
      
        return currentMax;
    },
    
    
    // difference is the current y-value - the y-value at the beginning of the range
    differenceForRange: function(minX, maxX)
    {
        var currentMaxX = -9999999;
        var currentMinX = 9999999;
        var beginningValue = null;
        var endingValue = null;
        
        var sampleArray = new Array();
        
        for ( var i = 0; i < this.points.length; i++ )
        {
            if ( this.points[i] == null ||
                this.points[i].yValue == null ||
                this.points[i].xValue == null ||
                isNaN(this.points[i].yValue) ||
                isNaN(this.points[i].xValue) ) continue;
            
            if ( this.points[i].xValue < minX || 
                this.points[i].xValue > maxX ) continue;
            
            sampleArray.push(this.points[i].yValue);
        }
        
        if ( sampleArray.length == 0 ) return null;
        
        if ( sampleArray.length == 1 ) return 0;
                
        return sampleArray[sampleArray.length-1] - sampleArray[0];
    },
    
    medianForRange: function(minX, maxX)
    {
        var sampleArray = new Array();
    
        for ( var i = 0; i < this.points.length; i++ )
        {
            if ( this.points[i] == null ||
                 this.points[i].yValue == null ||
                 this.points[i].xValue == null ||
                 isNaN(this.points[i].yValue) ||
                 isNaN(this.points[i].xValue) ) continue;
      
             if ( this.points[i].xValue < minX ||
                  this.points[i].xValue > maxX ) continue;
                
             sampleArray.push(this.points[i].yValue);
        }

        if ( sampleArray.length == 0 ) return null;
        
        if ( sampleArray.length == 1 ) return sampleArray[0];
        
        if ( sampleArray.length == 2 ) return ( sampleArray[0] + sampleArray[1] ) / 2
        
        sampleArray.sort(Utilities.compareNumbers);
    
        var middleIndex = Math.floor(sampleArray.length / 2);
        
        if ( sampleArray.length % 2 == 0 )  // even number of points, take average of middle two
        {
           var tmp = ( sampleArray[middleIndex] + sampleArray[middleIndex+1] ) / 2;
           return tmp;
        }
        
        return sampleArray[middleIndex];
    }
    
};

/*  Example pt (dataPoint) XML
  <pt>
    <y>44</y>
    <x>1234589845</x>
    <note>This is my peak for the year</note>
    <date>13483499</date>
    <by>84398</by>
    <ptStyle>
      <color>red</color>
    </ptStyle>
  </pt>
  <pt>
    <y>48</y>
    <x>1223239845</x>
  </pt>
  <pt>
    <y>66</y>
    <x>133239845</x>
  </pt>
*/
DataPoint = function()
{
  this.xValue = null;
  this.yValue = null;
  this.xDescription = null;
  this.yDescription = null;
  this.note = null;
  this.dateEntered = null;
  this.submitter = null;
  this.styleKeys = new Array();
  this.styleAttributes = new Array();
}

DataPoint.prototype =
{
    setValues: function(x, y, xDescrip, yDescrip)
    {
        this.xValue = x;
        this.yValue = y;
        this.xDescription = xDescrip;
        this.yDescription = yDescrip;
        return this;
    },
    
    extractFromDOM: function(XMLDOMObject)
    {
        var elements = XMLDOMObject.getElementsByTagName("x");
        if ( elements && elements.length > 0 && elements.item(0).firstChild ) this.xValue = parseInt(elements.item(0).firstChild.nodeValue);
        elements = XMLDOMObject.getElementsByTagName("y");
        if ( elements && elements.length > 0 && elements.item(0).firstChild ) this.yValue = parseFloat(elements.item(0).firstChild.nodeValue);
        var elements = XMLDOMObject.getElementsByTagName("xD");
        if ( elements && elements.length > 0 && elements.item(0).firstChild ) this.xDescription = elements.item(0).firstChild.nodeValue;
        elements = XMLDOMObject.getElementsByTagName("yD");
        if ( elements && elements.length > 0 && elements.item(0).firstChild ) this.yDescription = elements.item(0).firstChild.nodeValue;
        elements = XMLDOMObject.getElementsByTagName("note");
        if ( elements && elements.length > 0 && elements.item(0).firstChild ) this.note = elements.item(0).firstChild.nodeValue;
        elements = XMLDOMObject.getElementsByTagName("date");
        if ( elements && elements.length > 0 && elements.item(0).firstChild ) this.dateEntered = parseInt(elements.item(0).firstChild.nodeValue);
        elements = XMLDOMObject.getElementsByTagName("by");
        if ( elements && elements.length > 0 && elements.item(0).firstChild ) this.submitter = elements.item(0).firstChild.nodeValue;

        var DOMstyles = XMLDOMObject.getElementsByTagName("ptStyle");
        if ( DOMstyles != null && DOMstyles.length > 0)
        {
            var aStyleNode = DOMstyles.item(0).firstChild;
            while ( aStyleNode != null )
            {
              this.styleKeys.push(aStyleNode.nodeName);
              this.styleAttributes[aStyleNode.nodeName] = aStyleNode.firstChild.nodeValue;
              aStyleNode = aStyleNode.nextSibling;
            }
        }

        return this;
    },

    toXML: function()
    {
        var result = "<pt>";
        if ( this.xValue != null ) result += "<x>" + this.xValue + "</x>";
        if ( this.yValue != null ) result += "<y>" + this.yValue + "</y>";
        if ( this.xDescription != null ) result += "<xD>" + this.xDescription + "</xD>";
        if ( this.yDescription != null && this.yDescription != this.yValue ) result += "<yD>" + this.yDescription + "</yD>";
        if ( this.note != null ) result += "<note><![CDATA[" + this.note + "]]></note>";
        if ( this.dateEntered != null ) result += "<date>" + this.dateEntered + "</date>";
        if ( this.submitter != null ) result += "<by>" + this.submitter + "</by>";

        if ( this.styleKeys != null && this.styleKeys.length > 0 )
        {
           result += "<ptStyle>"
           for ( var i = 0; i < this.styleKeys.length; i++)
           {
               var aKey = this.styleKeys[i];
               result += "<" + aKey + ">" + this.styleAttributes[aKey] + "</" + aKey + ">";
           }
           result += "</ptStyle>";
        }

        result += "</pt>";
        
        return result;
    }

};


